@@ -54,7 +54,8 @@ static int format_convert(const SUNMatrix A, SUNMatrix B);
  */
 
 SUNMatrix SUNSparseMatrix(sunindextype M, sunindextype N,
-                          sunindextype NNZ, int sparsetype)
+                          sunindextype NNZ, int sparsetype,
+                          SUNContext sunctx)
 {
   SUNMatrix A;
   SUNMatrixContent_Sparse content;
@@ -65,7 +66,7 @@ SUNMatrix SUNSparseMatrix(sunindextype M, sunindextype N,
 
   /* Create an empty matrix object */
   A = NULL;
-  A = SUNMatNewEmpty();
+  A = SUNMatNewEmpty(sunctx);
   if (A == NULL) return(NULL);
 
   /* Attach operations */
@@ -136,7 +137,8 @@ SUNMatrix SUNSparseMatrix(sunindextype M, sunindextype N,
  * if the request for matrix storage cannot be satisfied.
  */
 
-SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol, int sparsetype)
+SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol,
+                                   int sparsetype)
 {
   sunindextype i, j, nnz;
   sunindextype M, N;
@@ -161,7 +163,8 @@ SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol, int sparsetyp
       nnz += (SUNRabs(SM_ELEMENT_D(Ad,i,j)) > droptol);
 
   /* allocate sparse matrix */
-  As = SUNSparseMatrix(M, N, nnz, sparsetype);
+  As = NULL;
+  As = SUNSparseMatrix(M, N, nnz, sparsetype, Ad->sunctx);
   if (As == NULL)  return NULL;
 
   /* copy nonzeros from Ad into As, based on CSR/CSC type */
@@ -225,7 +228,7 @@ SUNMatrix SUNSparseFromBandMatrix(SUNMatrix Ad, realtype droptol, int sparsetype
       nnz += (SUNRabs(SM_ELEMENT_B(Ad,i,j)) > droptol);
 
   /* allocate sparse matrix */
-  As = SUNSparseMatrix(M, N, nnz, sparsetype);
+  As = SUNSparseMatrix(M, N, nnz, sparsetype, Ad->sunctx);
   if (As == NULL)  return NULL;
 
   /* copy nonzeros from Ad into As, based on CSR/CSC type */
@@ -266,7 +269,7 @@ int SUNSparseMatrix_ToCSR(const SUNMatrix A, SUNMatrix* Bout)
     if (A == NULL) return(SUNMAT_ILL_INPUT);
     if (SM_SPARSETYPE_S(A) != CSC_MAT) return(SUNMAT_ILL_INPUT);
 
-    *Bout = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A), SM_NNZ_S(A), CSR_MAT);
+    *Bout = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A), SM_NNZ_S(A), CSR_MAT, A->sunctx);
     if (*Bout == NULL) return(SUNMAT_MEM_FAIL);
 
     return format_convert(A, *Bout);
@@ -281,7 +284,7 @@ int SUNSparseMatrix_ToCSC(const SUNMatrix A, SUNMatrix* Bout)
     if (A == NULL) return(SUNMAT_ILL_INPUT);
     if (SM_SPARSETYPE_S(A) != CSR_MAT) return(SUNMAT_ILL_INPUT);
 
-    *Bout = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A), SM_NNZ_S(A), CSC_MAT);
+    *Bout = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A), SM_NNZ_S(A), CSC_MAT, A->sunctx);
     if (*Bout == NULL) return(SUNMAT_MEM_FAIL);
 
     return format_convert(A, *Bout);
@@ -473,7 +476,7 @@ SUNMatrix_ID SUNMatGetID_Sparse(SUNMatrix A)
 SUNMatrix SUNMatClone_Sparse(SUNMatrix A)
 {
   SUNMatrix B = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A),
-                                SM_NNZ_S(A), SM_SPARSETYPE_S(A));
+                                SM_NNZ_S(A), SM_SPARSETYPE_S(A), A->sunctx);
   return(B);
 }
 
@@ -700,7 +703,7 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
     /* create new matrix for sum */
     C = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A),
                         Ap[N] + newvals,
-                        SM_SPARSETYPE_S(A));
+                        SM_SPARSETYPE_S(A), A->sunctx);
 
     /* access data from CSR structures (return if failure) */
     Cp = Ci = NULL;
@@ -924,7 +927,7 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
 
     /* create new matrix for sum */
     C = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A),
-                        Ap[N] + newvals, SM_SPARSETYPE_S(A));
+                        Ap[N] + newvals, SM_SPARSETYPE_S(A), A->sunctx);
 
     /* access data from CSR structures (return if failure) */
     Cp = Ci = NULL;
@@ -1180,7 +1183,7 @@ int format_convert(const SUNMatrix A, SUNMatrix B)
     Ap = SM_INDEXPTRS_S(A);
     Aj = SM_INDEXVALS_S(A);
     Ax = SM_DATA_S(A);
-    
+
     n_row = (SM_SPARSETYPE_S(A) == CSR_MAT) ? SM_ROWS_S(A) : SM_COLUMNS_S(A);
     n_col = (SM_SPARSETYPE_S(A) == CSR_MAT) ? SM_COLUMNS_S(A) : SM_ROWS_S(A);
 
@@ -1230,4 +1233,4 @@ int format_convert(const SUNMatrix A, SUNMatrix B)
     }
 
     return 0;
-}
\ No newline at end of file
+}
