@@ -67,13 +67,14 @@ class AgradCdfLogTest {
 template <class T>
 class AgradCdfLogTestFixture : public ::testing::Test {
  public:
-  typename at_c<T, 0>::type TestClass;
-  typedef typename at_c<typename at_c<T, 1>::type, 0>::type T0;
-  typedef typename at_c<typename at_c<T, 1>::type, 1>::type T1;
-  typedef typename at_c<typename at_c<T, 1>::type, 2>::type T2;
-  typedef typename at_c<typename at_c<T, 1>::type, 3>::type T3;
-  typedef typename at_c<typename at_c<T, 1>::type, 4>::type T4;
-  typedef typename at_c<typename at_c<T, 1>::type, 5>::type T5;
+  std::tuple_element_t<0, T> TestClass;
+  typedef std::tuple_element_t<1, T> ArgClass;
+  typedef std::tuple_element_t<0, ArgClass> T0;
+  typedef std::tuple_element_t<1, ArgClass> T1;
+  typedef std::tuple_element_t<2, ArgClass> T2;
+  typedef std::tuple_element_t<3, ArgClass> T3;
+  typedef std::tuple_element_t<4, ArgClass> T4;
+  typedef std::tuple_element_t<5, ArgClass> T5;
 
   typedef typename scalar_type<T0>::type Scalar0;
   typedef typename scalar_type<T1>::type Scalar1;
@@ -243,108 +244,131 @@ class AgradCdfLogTestFixture : public ::testing::Test {
   }
 
   // works for <double>
-  double calculate_gradients_1storder(vector<double>& grad, double& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log;
+  template <typename... Args>
+  double calculate_gradients_1storder(vector<double>& grad, double& logprob,
+                                      Args&... args) {
+    return logprob;
   }
-  double calculate_gradients_2ndorder(vector<double>& grad, double& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log;
+  template <typename... Args>
+  double calculate_gradients_2ndorder(vector<double>& grad, double& logprob,
+                                      Args&... x) {
+    return logprob;
   }
-  double calculate_gradients_3rdorder(vector<double>& grad, double& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log;
+  template <typename... Args>
+  double calculate_gradients_3rdorder(vector<double>& grad, double& logprob,
+                                      Args&... x) {
+    return logprob;
   }
 
   // works for <var>
-  double calculate_gradients_1storder(vector<double>& grad, var& cdf_log,
-                                      vector<var>& x) {
+  template <typename... Args>
+  double calculate_gradients_1storder(vector<double>& grad, var& logprob,
+                                      Args&... args) {
     stan::math::set_zero_all_adjoints();
-    cdf_log.grad(x, grad);
-    return cdf_log.val();
+    logprob.grad();
+    add_adjoints(grad, args...);
+    return logprob.val();
   }
-  double calculate_gradients_2ndorder(vector<double>& grad, var& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log.val();
+  template <typename... Args>
+  double calculate_gradients_2ndorder(vector<double>& grad, var& logprob,
+                                      Args&... args) {
+    return logprob.val();
   }
-  double calculate_gradients_3rdorder(vector<double>& grad, var& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log.val();
+  template <typename... Args>
+  double calculate_gradients_3rdorder(vector<double>& grad, var& logprob,
+                                      Args&... args) {
+    return logprob.val();
   }
 
   // works for fvar<double>
+  template <typename... Args>
   double calculate_gradients_1storder(vector<double>& grad,
-                                      fvar<double>& cdf_log, vector<var>& x) {
-    grad.push_back(cdf_log.d_);
-    return cdf_log.val();
+                                      fvar<double>& logprob, Args&... args) {
+    grad.push_back(logprob.d_);
+    return logprob.val();
   }
+  template <typename... Args>
   double calculate_gradients_2ndorder(vector<double>& grad,
-                                      fvar<double>& cdf_log, vector<var>& x) {
-    return cdf_log.val();
+                                      fvar<double>& logprob, Args&... args) {
+    return logprob.val();
   }
+  template <typename... Args>
   double calculate_gradients_3rdorder(vector<double>& grad,
-                                      fvar<double>& cdf_log, vector<var>& x) {
-    return cdf_log.val();
+                                      fvar<double>& logprob, Args&... args) {
+    return logprob.val();
   }
 
   // works for fvar<fvar<double> >
+  template <typename... Args>
   double calculate_gradients_1storder(vector<double>& grad,
-                                      fvar<fvar<double>>& cdf_log,
-                                      vector<var>& x) {
-    grad.push_back(cdf_log.d_.val_);
-    return cdf_log.val().val();
+                                      fvar<fvar<double>>& logprob,
+                                      Args&... args) {
+    grad.push_back(logprob.d_.val_);
+
+    return logprob.val().val();
   }
+  template <typename... Args>
   double calculate_gradients_2ndorder(vector<double>& grad,
-                                      fvar<fvar<double>>& cdf_log,
-                                      vector<var>& x) {
-    grad.push_back(cdf_log.d_.d_);
-    return cdf_log.val().val();
+                                      fvar<fvar<double>>& logprob,
+                                      Args&... args) {
+    grad.push_back(logprob.d_.d_);
+    return logprob.val().val();
   }
+  template <typename... Args>
   double calculate_gradients_3rdorder(vector<double>& grad,
-                                      fvar<fvar<double>>& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log.val().val();
+                                      fvar<fvar<double>>& logprob,
+                                      Args&... args) {
+    return logprob.val().val();
   }
 
   // works for fvar<var>
-  double calculate_gradients_1storder(vector<double>& grad, fvar<var>& cdf_log,
-                                      vector<var>& x) {
+  template <typename... Args>
+  double calculate_gradients_1storder(vector<double>& grad, fvar<var>& logprob,
+                                      Args&... args) {
     stan::math::set_zero_all_adjoints();
-    cdf_log.val_.grad(x, grad);
-    return cdf_log.val_.val();
+    logprob.val_.grad();
+    add_adjoints(grad, args...);
+    return logprob.val_.val();
   }
-  double calculate_gradients_2ndorder(vector<double>& grad, fvar<var>& cdf_log,
-                                      vector<var>& x) {
+  template <typename... Args>
+  double calculate_gradients_2ndorder(vector<double>& grad, fvar<var>& logprob,
+                                      Args&... args) {
     stan::math::set_zero_all_adjoints();
-    cdf_log.d_.grad(x, grad);
-    return cdf_log.val_.val();
+    logprob.d_.grad();
+    add_adjoints(grad, args...);
+    return logprob.val_.val();
   }
-  double calculate_gradients_3rdorder(vector<double>& grad, fvar<var>& cdf_log,
-                                      vector<var>& x) {
-    return cdf_log.val_.val();
+  template <typename... Args>
+  double calculate_gradients_3rdorder(vector<double>& grad, fvar<var>& logprob,
+                                      Args&... args) {
+    return logprob.val_.val();
   }
 
   // works for fvar<fvar<var> >
+  template <typename... Args>
   double calculate_gradients_1storder(vector<double>& grad,
-                                      fvar<fvar<var>>& cdf_log,
-                                      vector<var>& x) {
+                                      fvar<fvar<var>>& logprob, Args&... args) {
     stan::math::set_zero_all_adjoints();
-    cdf_log.val_.val_.grad(x, grad);
-    return cdf_log.val_.val_.val();
+    logprob.val_.val_.grad();
+    add_adjoints(grad, args...);
+    return logprob.val_.val_.val();
   }
+  template <typename... Args>
   double calculate_gradients_2ndorder(vector<double>& grad,
-                                      fvar<fvar<var>>& cdf_log,
-                                      vector<var>& x) {
+                                      fvar<fvar<var>>& logprob, Args&... args) {
     stan::math::set_zero_all_adjoints();
-    cdf_log.d_.val_.grad(x, grad);
-    return cdf_log.val_.val_.val();
+    logprob.d_.val_.grad();
+    add_adjoints(grad, args...);
+
+    return logprob.val_.val_.val();
   }
+  template <typename... Args>
   double calculate_gradients_3rdorder(vector<double>& grad,
-                                      fvar<fvar<var>>& cdf_log,
-                                      vector<var>& x) {
+                                      fvar<fvar<var>>& logprob, Args&... args) {
     stan::math::set_zero_all_adjoints();
-    cdf_log.d_.d_.grad(x, grad);
-    return cdf_log.val_.val_.val();
+    logprob.d_.d_.grad();
+    add_adjoints(grad, args...);
+    return logprob.val_.val_.val();
   }
 
   void test_finite_diffs_equal(const vector<double>& parameters,
@@ -399,15 +423,13 @@ class AgradCdfLogTestFixture : public ::testing::Test {
         Scalar4 p4 = get_param<Scalar4>(parameters[n], 4);
         Scalar5 p5 = get_param<Scalar5>(parameters[n], 5);
 
-        vector<var> x1;
-        add_vars(x1, p0, p1, p2, p3, p4, p5);
-
         T_return_type cdf_log
             = TestClass.template cdf_log<Scalar0, Scalar1, Scalar2, Scalar3,
                                          Scalar4, Scalar5>(p0, p1, p2, p3, p4,
                                                            p5);
 
-        calculate_gradients_1storder(gradients, cdf_log, x1);
+        calculate_gradients_1storder(gradients, cdf_log, p0, p1, p2, p3, p4,
+                                     p5);
 
         test_finite_diffs_equal(parameters[n], finite_diffs, gradients);
 
@@ -455,19 +477,6 @@ class AgradCdfLogTestFixture : public ::testing::Test {
       Scalar4 p4 = get_param<Scalar4>(parameters[n], 4);
       Scalar5 p5 = get_param<Scalar5>(parameters[n], 5);
 
-      vector<var> x1;
-      vector<var> x2;
-      vector<var> x3;
-      vector<var> y1;
-      vector<var> y2;
-      vector<var> y3;
-      add_vars(x1, p0, p1, p2, p3, p4, p5);
-      add_vars(x2, p0, p1, p2, p3, p4, p5);
-      add_vars(x3, p0, p1, p2, p3, p4, p5);
-      add_vars(y1, p0, p1, p2, p3, p4, p5);
-      add_vars(y2, p0, p1, p2, p3, p4, p5);
-      add_vars(y3, p0, p1, p2, p3, p4, p5);
-
       T_return_type cdf_log
           = TestClass.template cdf_log<Scalar0, Scalar1, Scalar2, Scalar3,
                                        Scalar4, Scalar5>(p0, p1, p2, p3, p4,
@@ -478,12 +487,15 @@ class AgradCdfLogTestFixture : public ::testing::Test {
                                                 Scalar3, Scalar4, Scalar5>(
               p0, p1, p2, p3, p4, p5);
 
-      calculate_gradients_1storder(expected_gradients1, cdf_log_funct, x1);
-      calculate_gradients_1storder(gradients1, cdf_log, y1);
-      calculate_gradients_2ndorder(expected_gradients2, cdf_log_funct, x2);
-      calculate_gradients_2ndorder(gradients2, cdf_log, y2);
-      calculate_gradients_3rdorder(expected_gradients3, cdf_log_funct, x3);
-      calculate_gradients_3rdorder(gradients3, cdf_log, y3);
+      calculate_gradients_1storder(expected_gradients1, cdf_log_funct, p0, p1,
+                                   p2, p3, p4, p5);
+      calculate_gradients_1storder(gradients1, cdf_log, p0, p1, p2, p3, p4, p5);
+      calculate_gradients_2ndorder(expected_gradients2, cdf_log_funct, p0, p1,
+                                   p2, p3, p4, p5);
+      calculate_gradients_2ndorder(gradients2, cdf_log, p0, p1, p2, p3, p4, p5);
+      calculate_gradients_3rdorder(expected_gradients3, cdf_log_funct, p0, p1,
+                                   p2, p3, p4, p5);
+      calculate_gradients_3rdorder(gradients3, cdf_log, p0, p1, p2, p3, p4, p5);
 
       test_gradients_equal(expected_gradients1, gradients1);
       test_gradients_equal(expected_gradients2, gradients2);
@@ -500,6 +512,17 @@ class AgradCdfLogTestFixture : public ::testing::Test {
    * For log cdf this means lcdf([a, a, a]) == lcdf(a) + lcdf(a) + lcdf(a)
    */
   void test_repeat_as_vector() {
+    if (stan::is_any_var_matrix<T0, T1, T2, T3, T4, T5>::value) {
+      // There is no way to do this test for a `var_value` matrix
+      // because this is testing what happens when all elements of
+      // the vector are the same thing. This works the PIMP var types
+      // stored in other containers because every var can point at the
+      // same vari. However, when a var_value of length N is allocated
+      // there are N values and N adjoints and they are all separate.
+
+      SUCCEED() << "No test for var_value<Eigen::Matrix<T, R, C>> arguments";
+      return;
+    }
     if (!any_vector<T0, T1, T2, T3, T4, T5>::value) {
       SUCCEED() << "No test for non-vector arguments";
       return;
@@ -520,26 +543,22 @@ class AgradCdfLogTestFixture : public ::testing::Test {
       Scalar3 p3_ = get_param<Scalar3>(parameters[n], 3);
       Scalar4 p4_ = get_param<Scalar4>(parameters[n], 4);
       Scalar5 p5_ = get_param<Scalar5>(parameters[n], 5);
-      vector<var> s1;
-      vector<var> s2;
-      vector<var> s3;
       std::vector<Scalar0> p0s_((is_vector<T0>::value) ? N_REPEAT : 1, p0_);
       std::vector<Scalar1> p1s_((is_vector<T1>::value) ? N_REPEAT : 1, p1_);
       std::vector<Scalar2> p2s_((is_vector<T2>::value) ? N_REPEAT : 1, p2_);
       std::vector<Scalar3> p3s_((is_vector<T3>::value) ? N_REPEAT : 1, p3_);
       std::vector<Scalar4> p4s_((is_vector<T4>::value) ? N_REPEAT : 1, p4_);
       std::vector<Scalar5> p5s_((is_vector<T5>::value) ? N_REPEAT : 1, p5_);
-      add_vars(s1, p0s_, p1s_, p2s_, p3s_, p4s_, p5s_);
-      add_vars(s2, p0s_, p1s_, p2s_, p3s_, p4s_, p5s_);
-      add_vars(s3, p0s_, p1s_, p2s_, p3s_, p4s_, p5s_);
 
       T_return_type cdf_log
           = N_REPEAT * TestClass.cdf_log(p0_, p1_, p2_, p3_, p4_, p5_);
 
-      double single_cdf_log
-          = calculate_gradients_1storder(single_gradients1, cdf_log, s1);
-      calculate_gradients_2ndorder(single_gradients2, cdf_log, s2);
-      calculate_gradients_3rdorder(single_gradients3, cdf_log, s3);
+      double single_cdf_log = calculate_gradients_1storder(
+          single_gradients1, cdf_log, p0s_, p1s_, p2s_, p3s_, p4s_, p5s_);
+      calculate_gradients_2ndorder(single_gradients2, cdf_log, p0s_, p1s_, p2s_,
+                                   p3s_, p4s_, p5s_);
+      calculate_gradients_3rdorder(single_gradients3, cdf_log, p0s_, p1s_, p2s_,
+                                   p3s_, p4s_, p5s_);
 
       T0 p0 = get_repeated_params<T0>(parameters[n], 0, N_REPEAT);
       T1 p1 = get_repeated_params<T1>(parameters[n], 1, N_REPEAT);
@@ -553,16 +572,13 @@ class AgradCdfLogTestFixture : public ::testing::Test {
       vector<double> multiple_gradients1;
       vector<double> multiple_gradients2;
       vector<double> multiple_gradients3;
-      vector<var> x1;
-      vector<var> x2;
-      vector<var> x3;
-      add_vars(x1, p0, p1, p2, p3, p4, p5);
-      add_vars(x2, p0, p1, p2, p3, p4, p5);
-      add_vars(x3, p0, p1, p2, p3, p4, p5);
 
-      calculate_gradients_1storder(multiple_gradients1, multiple_cdf_log, x1);
-      calculate_gradients_2ndorder(multiple_gradients2, multiple_cdf_log, x2);
-      calculate_gradients_3rdorder(multiple_gradients3, multiple_cdf_log, x3);
+      calculate_gradients_1storder(multiple_gradients1, multiple_cdf_log, p0,
+                                   p1, p2, p3, p4, p5);
+      calculate_gradients_2ndorder(multiple_gradients2, multiple_cdf_log, p0,
+                                   p1, p2, p3, p4, p5);
+      calculate_gradients_3rdorder(multiple_gradients3, multiple_cdf_log, p0,
+                                   p1, p2, p3, p4, p5);
 
       stan::math::recover_memory();
 
@@ -605,12 +621,10 @@ class AgradCdfLogTestFixture : public ::testing::Test {
     vector<double> single_gradients1;
     vector<double> single_gradients2;
     vector<double> single_gradients3;
-    vector<var> scalar_vars;
 
     vector<double> multiple_gradients1;
     vector<double> multiple_gradients2;
     vector<double> multiple_gradients3;
-    vector<var> vector_vars;
 
     T0 p0 = get_params<T0>(parameters, 0);
     T1 p1 = get_params<T1>(parameters, 1);
@@ -652,39 +666,21 @@ class AgradCdfLogTestFixture : public ::testing::Test {
                                           p3s.back(), p4s.back(), p5s.back());
     }
 
-    add_var(vector_vars, p0);
-    add_var(scalar_vars, p0s);
-
-    add_var(vector_vars, p1);
-    add_var(scalar_vars, p1s);
-
-    add_var(vector_vars, p2);
-    add_var(scalar_vars, p2s);
-
-    add_var(vector_vars, p3);
-    add_var(scalar_vars, p3s);
-
-    add_var(vector_vars, p4);
-    add_var(scalar_vars, p4s);
-
-    add_var(vector_vars, p5);
-    add_var(scalar_vars, p5s);
-
-    calculate_gradients_1storder(single_gradients1, single_cdf_log,
-                                 scalar_vars);
-    calculate_gradients_2ndorder(single_gradients2, single_cdf_log,
-                                 scalar_vars);
-    calculate_gradients_3rdorder(single_gradients3, single_cdf_log,
-                                 scalar_vars);
+    calculate_gradients_1storder(single_gradients1, single_cdf_log, p0s, p1s,
+                                 p2s, p3s, p4s, p5s);
+    calculate_gradients_2ndorder(single_gradients2, single_cdf_log, p0s, p1s,
+                                 p2s, p3s, p4s, p5s);
+    calculate_gradients_3rdorder(single_gradients3, single_cdf_log, p0s, p1s,
+                                 p2s, p3s, p4s, p5s);
 
     T_return_type multiple_cdf_log = TestClass.cdf_log(p0, p1, p2, p3, p4, p5);
 
-    calculate_gradients_1storder(multiple_gradients1, multiple_cdf_log,
-                                 vector_vars);
-    calculate_gradients_2ndorder(multiple_gradients2, multiple_cdf_log,
-                                 vector_vars);
-    calculate_gradients_3rdorder(multiple_gradients3, multiple_cdf_log,
-                                 vector_vars);
+    calculate_gradients_1storder(multiple_gradients1, multiple_cdf_log, p0, p1,
+                                 p2, p3, p4, p5);
+    calculate_gradients_2ndorder(multiple_gradients2, multiple_cdf_log, p0, p1,
+                                 p2, p3, p4, p5);
+    calculate_gradients_3rdorder(multiple_gradients3, multiple_cdf_log, p0, p1,
+                                 p2, p3, p4, p5);
 
     stan::math::recover_memory();
 
