@@ -107,7 +107,7 @@ TEST(ModelIndexing, lvalueUniUni) {
   xs1.push_back(1.1);
   xs1.push_back(1.2);
 
-  vector<vector<double> > xs;
+  vector<vector<double>> xs;
   xs.push_back(xs0);
   xs.push_back(xs1);
 
@@ -198,7 +198,9 @@ TEST(ModelIndexing, lvalueMultiEigen) {
   EXPECT_FLOAT_EQ(y[1], x[1]);
   EXPECT_FLOAT_EQ(2, x[2]);
   test_throw_ia(x, y, index_max(10));
-
+  test_throw_ia(x, y, index_max(0));
+  Eigen::VectorXd z_empty(0);
+  EXPECT_NO_THROW(assign(z_empty, Eigen::VectorXd(0), "", index_max(0)));
   vector<int> ns;
   ns.push_back(4);
   ns.push_back(6);
@@ -214,10 +216,16 @@ TEST(ModelIndexing, lvalueMultiEigen) {
 
   ns.push_back(3);
   test_throw_ia(x, y, index_multi(ns));
+
+  std::vector<double> z_std_empty(0);
+  EXPECT_NO_THROW(
+      assign(z_std_empty, std::vector<double>{}, "", index_max(-5)));
+  EXPECT_NO_THROW(
+      assign(z_std_empty, std::vector<double>{}, "", index_min_max(1, -1)));
 }
 
 TEST(ModelIndexing, lvalueMultiMulti) {
-  vector<vector<double> > xs;
+  vector<vector<double>> xs;
   for (int i = 0; i < 10; ++i) {
     vector<double> xsi;
     for (int j = 0; j < 20; ++j)
@@ -225,7 +233,7 @@ TEST(ModelIndexing, lvalueMultiMulti) {
     xs.push_back(xsi);
   }
 
-  vector<vector<double> > ys;
+  vector<vector<double>> ys;
   for (int i = 0; i < 2; ++i) {
     vector<double> ysi;
     for (int j = 0; j < 3; ++j)
@@ -241,6 +249,12 @@ TEST(ModelIndexing, lvalueMultiMulti) {
 
   test_throw_ia(xs, ys, index_min(7), index_max(3));
   test_throw_ia(xs, ys, index_min(9), index_max(2));
+  std::vector<std::vector<double>> lhs_empty(11, std::vector<double>{});
+  std::vector<std::vector<double>> rhs_empty(3, std::vector<double>{});
+  EXPECT_NO_THROW(
+      assign(lhs_empty, rhs_empty, "", index_min(9), index_max(-4)));
+  EXPECT_NO_THROW(
+      assign(lhs_empty, rhs_empty, "", index_min(9), index_min_max(3, -4)));
 }
 
 TEST(ModelIndexing, lvalueMultiMultiEigen) {
@@ -270,10 +284,22 @@ TEST(ModelIndexing, lvalueMultiMultiEigen) {
 
   test_throw_ia(xs, ys, index_min(7), index_max(3));
   test_throw_ia(xs, ys, index_min(9), index_max(2));
+  std::vector<Eigen::VectorXd> lhs_inner_empty(11, Eigen::VectorXd(0));
+  std::vector<Eigen::VectorXd> rhs_inner_empty(3, Eigen::VectorXd(0));
+  EXPECT_NO_THROW(assign(lhs_inner_empty, rhs_inner_empty, "", index_min(9),
+                         index_min_max(1, -4)));
+  EXPECT_NO_THROW(assign(lhs_inner_empty, rhs_inner_empty, "", index_min(9),
+                         index_max(-4)));
+  std::vector<Eigen::VectorXd> lhs_empty(0, Eigen::VectorXd(0));
+  std::vector<Eigen::VectorXd> rhs_empty(0);
+  EXPECT_NO_THROW(
+      assign(lhs_empty, rhs_empty, "", index_max(-4), index_min(9)));
+  EXPECT_NO_THROW(
+      assign(lhs_empty, rhs_empty, "", index_min_max(3, -4), index_min(9)));
 }
 
 TEST(ModelIndexing, lvalueUniMulti) {
-  vector<vector<double> > xs;
+  vector<vector<double>> xs;
   for (int i = 0; i < 10; ++i) {
     vector<double> xsi;
     for (int j = 0; j < 20; ++j)
@@ -296,7 +322,7 @@ TEST(ModelIndexing, lvalueUniMulti) {
 }
 
 TEST(ModelIndexing, lvalueMultiUni) {
-  vector<vector<double> > xs;
+  vector<vector<double>> xs;
   for (int i = 0; i < 10; ++i) {
     vector<double> xsi;
     for (int j = 0; j < 20; ++j)
@@ -452,14 +478,19 @@ TEST(ModelIndexing, lvalueMatrixMax) {
   for (int i = 0; i < 2; ++i)
     for (int j = 0; j < 4; ++j)
       EXPECT_FLOAT_EQ(y(i, j), x(i, j));
-  test_throw(x, y, index_max(0));
+
   test_throw(x, y, index_max(8));
+  test_throw_ia(x, y, index_max(0));
   test_throw_ia(x, y, index_max(1));
 
   MatrixXd z(1, 2);
   z << 10, 20;
   test_throw_ia(x, z, index_max(1));
   test_throw_ia(x, z, index_max(2));
+  Eigen::MatrixXd z_empty(0, 4);
+  EXPECT_NO_THROW(
+      assign(z_empty, Eigen::MatrixXd(0, 4), "empty", index_max(-8)));
+  test_throw(z_empty, Eigen::MatrixXd(0, 2), index_min(-8));
 }
 
 TEST(ModelIndexing, lvalueMatrixUniUni) {
@@ -534,14 +565,9 @@ TEST(ModelIndexing, lvalueMatrixNegativeMinMaxRow) {
       3.2, 3.3, 4.0, 4.1, 4.2, 4.3;
 
   MatrixXd y = MatrixXd::Ones(3, 4);
-  assign(x, y, "", index_min_max(3, 1));
-  for (int i = 0; i < 3; ++i) {
-    for (int j = 0; j < 4; ++j) {
-      EXPECT_EQ(x(i, j), y(i, j));
-    }
-  }
+  test_throw_ia(x, y, index_min_max(3, 1));
   test_throw(x, y, index_min_max(2, 6));
-  test_throw(x, y, index_min_max(1, 0));
+  test_throw_ia(x, y, index_min_max(1, 0));
   test_throw_ia(x, y, index_min_max(2, 1));
 }
 
@@ -637,11 +663,11 @@ TEST(ModelIndexing, doubleToVar) {
   xs.push_back(1);
   xs.push_back(2);
   xs.push_back(3);
-  vector<vector<double> > xss;
+  vector<vector<double>> xss;
   xss.push_back(xs);
 
   vector<var> ys(3);
-  vector<vector<var> > yss;
+  vector<vector<var>> yss;
   yss.push_back(ys);
 
   assign(yss, xss, "foo", index_omni());
@@ -685,7 +711,7 @@ TEST(ModelIndexing, resultSizeNegIndexing) {
   rhs.push_back(-125);
 
   vector<double> lhs;
-  assign(rhs, lhs, "", index_min_max(1, 0));
+  test_throw_ia(lhs, rhs, index_min_max(1, 0));
   EXPECT_EQ(0, lhs.size());
 }
 
@@ -713,12 +739,15 @@ TEST(ModelIndexing, resultSizeNegIndexingEigen) {
   lhs << 1, 2, 3, 4, 5;
   Eigen::VectorXd rhs(4);
   rhs << 1, 2, 3, 4;
+  /*
   assign(lhs, rhs, "", index_min_max(4, 1));
   EXPECT_FLOAT_EQ(lhs(0), 4);
   EXPECT_FLOAT_EQ(lhs(1), 3);
   EXPECT_FLOAT_EQ(lhs(2), 2);
   EXPECT_FLOAT_EQ(lhs(3), 1);
   EXPECT_FLOAT_EQ(lhs(4), 5);
+  */
+  test_throw_ia(lhs, rhs, index_min_max(4, 1));
 }
 
 TEST(ModelIndexing, resultSizePosMinMaxPosMinMaxEigenMatrix) {
@@ -758,7 +787,10 @@ TEST(ModelIndexing, resultSizePosMinMaxNegMinMaxEigenMatrix) {
     x_rev(i) = x.size() - i - 1;
   }
 
-  for (int i = 0; i < x.rows(); ++i) {
+  for (int i = 1; i < x.rows(); ++i) {
+    test_throw_ia(x, x_rev.block(0, 0, i + 1, i + 1), index_min_max(1, i + 1),
+                  index_min_max(i + 1, 1));
+    /*
     Eigen::MatrixXd x_rowwise_reverse
         = x_rev.block(0, 0, i + 1, i + 1).rowwise().reverse();
     assign(x, x_rev.block(0, 0, i + 1, i + 1), "", index_min_max(1, i + 1),
@@ -771,6 +803,7 @@ TEST(ModelIndexing, resultSizePosMinMaxNegMinMaxEigenMatrix) {
     for (int j = 0; j < x.size(); ++j) {
       x(j) = j;
     }
+    */
   }
 }
 
@@ -785,7 +818,10 @@ TEST(ModelIndexing, resultSizeNigMinMaxPosMinMaxEigenMatrix) {
     x_rev(i) = x.size() - i - 1;
   }
 
-  for (int i = 0; i < x.rows(); ++i) {
+  for (int i = 1; i < x.rows(); ++i) {
+    test_throw_ia(x, x_rev.block(0, 0, i + 1, i + 1), index_min_max(i + 1, 1),
+                  index_min_max(1, i + 1));
+    /*
     Eigen::MatrixXd x_colwise_reverse
         = x_rev.block(0, 0, i + 1, i + 1).colwise().reverse();
     assign(x, x_rev.block(0, 0, i + 1, i + 1), "", index_min_max(i + 1, 1),
@@ -798,6 +834,7 @@ TEST(ModelIndexing, resultSizeNigMinMaxPosMinMaxEigenMatrix) {
     for (int j = 0; j < x.size(); ++j) {
       x(j) = j;
     }
+    */
   }
 }
 
@@ -812,7 +849,10 @@ TEST(ModelIndexing, resultSizeNegMinMaxNegMinMaxEigenMatrix) {
     x_rev(i) = x.size() - i - 1;
   }
 
-  for (int i = 0; i < x.rows(); ++i) {
+  for (int i = 1; i < x.rows(); ++i) {
+    test_throw_ia(x, x_rev.block(0, 0, i + 1, i + 1), index_min_max(i + 1, 1),
+                  index_min_max(i + 1, 1));
+    /*
     Eigen::MatrixXd x_reverse = x_rev.block(0, 0, i + 1, i + 1).reverse();
     assign(x, x_rev.block(0, 0, i + 1, i + 1), "", index_min_max(i + 1, 1),
            index_min_max(i + 1, 1));
@@ -824,6 +864,7 @@ TEST(ModelIndexing, resultSizeNegMinMaxNegMinMaxEigenMatrix) {
     for (int j = 0; j < x.size(); ++j) {
       x(j) = j;
     }
+    */
   }
 }
 
@@ -834,7 +875,7 @@ TEST(modelIndexing, doubleToVarSimple) {
 
   mat_d a(2, 2);
   a << 1, 2, 3, 4;
-  mat_v b;
+  mat_v b(2, 2);
   assign(b, a, "");
   for (int i = 0; i < a.size(); ++i)
     EXPECT_FLOAT_EQ(a(i), b(i).val());
