@@ -54,12 +54,13 @@ inline bool check_multi_duplicate(const stan::arena_t<std::vector<int>>& x_idx,
 
 template <typename T1, typename T2, typename... I>
 void test_throw_out_of_range(T1& lhs, const T2& rhs, const I&... idxs) {
-  EXPECT_THROW(stan::model::assign(lhs, rhs, "", idxs...), std::out_of_range);
+  EXPECT_THROW(stan::model::assign(lhs, rhs, "rhs", idxs...),
+               std::out_of_range);
 }
 
 template <typename T1, typename T2, typename... I>
 void test_throw_invalid_arg(T1& lhs, const T2& rhs, const I&... idxs) {
-  EXPECT_THROW(stan::model::assign(lhs, rhs, "", idxs...),
+  EXPECT_THROW(stan::model::assign(lhs, rhs, "rhs", idxs...),
                std::invalid_argument);
 }
 
@@ -289,7 +290,7 @@ void test_max_vec() {
   auto check_i = [](int i) { return i > 1; };
   check_adjs(check_i, x, "lhs");
   check_adjs([](int /* i */) { return true; }, y, "rhs");
-  test_throw_out_of_range(x, y, index_max(0));
+  test_throw_invalid_arg(x, y, index_max(0));
   test_throw_out_of_range(x, y, index_max(6));
   test_throw_invalid_arg(x, conditionally_generate_linear_var_vector<Vec>(3),
                          index_max(2));
@@ -356,19 +357,9 @@ void test_negative_minmax_varvector() {
   Vec x_val = x.val();
   auto y = conditionally_generate_linear_var_vector<Vec, RhsScalar>(4, 10);
 
-  assign(x, y, "", index_min_max(4, 1));
-  EXPECT_FLOAT_EQ(x.val()(0), 13);
-  EXPECT_FLOAT_EQ(x.val()(1), 12);
-  EXPECT_FLOAT_EQ(x.val()(2), 11);
-  EXPECT_FLOAT_EQ(x.val()(3), 10);
-  EXPECT_FLOAT_EQ(x.val()(4), 4);
-  sum(x).grad();
-  EXPECT_MATRIX_EQ(x.val(), x_val);
-  auto check_i = [](int i) { return (i > 3); };
-  check_adjs(check_i, x, "lhs");
-  check_adjs([](int /* i */) { return true; }, y, "rhs");
-  test_throw_out_of_range(x, y, index_min_max(3, 0));
-  test_throw_out_of_range(x, y, index_min_max(6, 1));
+  test_throw_invalid_arg(x, y, index_min_max(4, 1));
+  test_throw_invalid_arg(x, y, index_min_max(3, 0));
+  test_throw_invalid_arg(x, y, index_min_max(6, 1));
   test_throw_invalid_arg(x, conditionally_generate_linear_var_vector<Vec>(5),
                          index_min_max(4, 1));
   test_throw_invalid_arg(x, conditionally_generate_linear_var_vector<Vec>(3),
@@ -1178,7 +1169,7 @@ void max_matrix_test() {
   auto check_all = [](int j) { return true; };
   check_adjs(check_i_x, check_all, x, "lhs", 0);
   check_adjs(check_all, y, "rhs");
-  test_throw_out_of_range(x, y, index_max(0));
+  test_throw_invalid_arg(x, y, index_max(0));
   test_throw_out_of_range(x, y, index_max(4));
   test_throw_invalid_arg(x, y, index_max(1));
   var_value<MatrixXd> z(MatrixXd::Ones(1, 2));
@@ -1212,7 +1203,7 @@ void min_max_matrix_test() {
   check_adjs(check_all, check_all, y, "rhs");
   test_throw_out_of_range(x, y, index_min(0), index_max(2));
   test_throw_out_of_range(x, y, index_min(5), index_max(2));
-  test_throw_out_of_range(x, y, index_min(2), index_max(0));
+  test_throw_invalid_arg(x, y, index_min(2), index_max(0));
   test_throw_out_of_range(x, y, index_min(2), index_max(5));
   test_throw_invalid_arg(x, y, index_min(2), index_max(1));
   var_value<MatrixXd> z(MatrixXd::Ones(1, 4));
@@ -1279,27 +1270,18 @@ void negative_minmax_matrix_test() {
     x_rev_val(i) = x_val.size() - i - 1;
   }
 
-  for (int i = 0; i < x_val.rows(); ++i) {
+  for (int i = 1; i < x_val.rows(); ++i) {
     var_value<Eigen::MatrixXd> x(x_val);
     std::conditional_t<stan::is_var<RhsScalar>::value,
                        var_value<Eigen::MatrixXd>, Eigen::MatrixXd>
         x_rev(x_rev_val);
     const int ii = i + 1;
-    assign(x, x_rev.block(0, 0, ii, 5), "", index_min_max(ii, 1));
-    auto x_val_check = x.val().block(0, 0, ii, 5);
-    auto x_rev_val_check
-        = stan::math::value_of(x_rev).block(0, 0, ii, 5).colwise().reverse();
-    EXPECT_MATRIX_EQ(x_val_check, x_rev_val_check);
-    sum(x).grad();
-    auto check_i = [i](int kk) { return kk <= i; };
-    auto check_all = [i](int jj) { return true; };
-    check_adjs(check_i, check_all, x, "lhs", 0);
-    check_adjs(check_i, check_all, x_rev, "rhs", 1);
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, 5), index_min_max(ii, 0));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, 5),
-                            index_min_max(ii + x.rows(), 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, 5), index_min_max(ii, 2));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, 4), index_min_max(1, ii));
+    EXPECT_NO_THROW(
+        assign(x, x_rev.block(0, 0, 0, 5), "", index_min_max(ii, 1)));
+    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, 5), index_min_max(ii, 1));
+    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, 5), index_min_max(ii, 0));
+    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, 5),
+                           index_min_max(ii + x.rows(), 1));
     stan::math::recover_memory();
   }
 }
@@ -1337,6 +1319,7 @@ void positive_minmax_positive_minmax_matrix_test() {
     auto check_j = [i](int jj) { return jj <= i; };
     check_adjs(check_i, check_j, x, "lhs", 0);
     check_adjs(check_i, check_j, x_rev, "rhs", 1);
+
     test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(0, ii),
                             index_min_max(1, ii));
     test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(1, ii),
@@ -1344,8 +1327,10 @@ void positive_minmax_positive_minmax_matrix_test() {
     test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii),
                             index_min_max(1, x.rows() + 1),
                             index_min_max(1, ii));
+
     test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(1, ii),
                             index_min_max(1, x.rows() + 1));
+
     // We don't want to go out of bounds when making the eigen block.
     auto ii_range_high = ii == 5 ? 4 : ii + 1;
     test_throw_invalid_arg(x, x_rev.block(0, 0, ii - 1, ii),
@@ -1356,6 +1341,7 @@ void positive_minmax_positive_minmax_matrix_test() {
                            index_min_max(1, ii), index_min_max(1, ii));
     test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii_range_high),
                            index_min_max(1, ii), index_min_max(1, ii));
+
     stan::math::recover_memory();
   }
 }
@@ -1378,43 +1364,14 @@ void positive_minmax_negative_minmax_matrix_test() {
     x_rev_val(i) = x_val.size() - i - 1;
   }
 
-  for (int i = 0; i < x_val.rows(); ++i) {
+  for (int i = 1; i < x_val.rows(); ++i) {
     var_value<Eigen::MatrixXd> x(x_val);
     std::conditional_t<stan::is_var<RhsScalar>::value,
                        var_value<Eigen::MatrixXd>, Eigen::MatrixXd>
         x_rev(x_rev_val);
     const int ii = i + 1;
-    assign(x, x_rev.block(0, 0, ii, ii), "", index_min_max(1, ii),
-           index_min_max(ii, 1));
-    auto x_val_check = x.val().block(0, 0, ii, ii);
-    auto x_rev_val_check
-        = stan::math::value_of(x_rev).block(0, 0, ii, ii).rowwise().reverse();
-    EXPECT_MATRIX_EQ(x_val_check, x_rev_val_check);
-    sum(x).grad();
-    auto check_i = [i](int kk) { return kk <= i; };
-    auto check_j = [i](int jj) { return jj <= i; };
-    check_adjs(check_i, check_j, x, "lhs", 0);
-    check_adjs(check_i, check_j, x_rev, "rhs", 1);
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(0, ii),
-                            index_min_max(ii, 1));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(1, ii),
-                            index_min_max(ii, 0));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii),
-                            index_min_max(1, x.rows() + 1),
-                            index_min_max(1, ii));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(1, ii),
-                            index_min_max(1, x.rows() + 1));
-    // We don't want to go out of bounds when making the eigen block.
-    auto ii_range_high = ii == 5 ? 4 : ii + 1;
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii - 1, ii),
-                           index_min_max(1, ii), index_min_max(ii, 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii_range_high, ii),
-                           index_min_max(1, ii), index_min_max(ii, 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii - 1),
-                           index_min_max(1, ii), index_min_max(ii, 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii_range_high),
-                           index_min_max(1, ii), index_min_max(ii, 1));
-    stan::math::recover_memory();
+    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii), index_min_max(1, ii),
+                           index_min_max(ii, 1));
   }
 }
 
@@ -1436,42 +1393,14 @@ void negative_minmax_positive_minmax_matrix_test() {
     x_rev_val(i) = x_val.size() - i - 1;
   }
 
-  for (int i = 0; i < x_val.rows(); ++i) {
+  for (int i = 1; i < x_val.rows(); ++i) {
     var_value<Eigen::MatrixXd> x(x_val);
     std::conditional_t<stan::is_var<RhsScalar>::value,
                        var_value<Eigen::MatrixXd>, Eigen::MatrixXd>
         x_rev(x_rev_val);
     const int ii = i + 1;
-    assign(x, x_rev.block(0, 0, ii, ii), "", index_min_max(ii, 1),
-           index_min_max(1, ii));
-    auto x_val_check = x.val().block(0, 0, ii, ii);
-    auto x_rev_val_check
-        = stan::math::value_of(x_rev).block(0, 0, ii, ii).colwise().reverse();
-    EXPECT_MATRIX_EQ(x_val_check, x_rev_val_check);
-    sum(x).grad();
-    auto check_i = [i](int kk) { return kk <= i; };
-    auto check_j = [i](int jj) { return jj <= i; };
-    check_adjs(check_i, check_j, x, "lhs", 0);
-    check_adjs(check_i, check_j, x_rev, "rhs", 1);
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 0),
-                            index_min_max(1, ii));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 1),
-                            index_min_max(0, ii));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii),
-                            index_min_max(x.rows() + 1, 1),
-                            index_min_max(1, ii));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 1),
-                            index_min_max(1, x.rows() + 1));
-    // We don't want to go out of bounds when making the eigen block.
-    auto ii_range_high = ii == 5 ? 4 : ii + 1;
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii - 1, ii),
-                           index_min_max(ii, 1), index_min_max(1, ii));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii_range_high, ii),
-                           index_min_max(ii, 1), index_min_max(1, ii));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii - 1),
-                           index_min_max(ii, 1), index_min_max(1, ii));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii_range_high),
-                           index_min_max(ii, 1), index_min_max(1, ii));
+    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 1),
+                           index_min_max(1, ii));
     stan::math::recover_memory();
   }
 }
@@ -1494,42 +1423,14 @@ void negative_minmax_negative_minmax_matrix_test() {
     x_rev_val(i) = x_val.size() - i - 1;
   }
 
-  for (int i = 0; i < x_val.rows(); ++i) {
+  for (int i = 1; i < x_val.rows(); ++i) {
     var_value<Eigen::MatrixXd> x(x_val);
     std::conditional_t<stan::is_var<RhsScalar>::value,
                        var_value<Eigen::MatrixXd>, Eigen::MatrixXd>
         x_rev(x_rev_val);
     const int ii = i + 1;
-    assign(x, x_rev.block(0, 0, ii, ii), "", index_min_max(ii, 1),
-           index_min_max(ii, 1));
-    auto x_val_check = x.val().block(0, 0, ii, ii);
-    auto x_rev_val_check
-        = stan::math::value_of(x_rev).block(0, 0, ii, ii).reverse();
-    EXPECT_MATRIX_EQ(x_val_check, x_rev_val_check);
-    sum(x).grad();
-    auto check_i = [i](int kk) { return kk <= i; };
-    auto check_j = [i](int jj) { return jj <= i; };
-    check_adjs(check_i, check_j, x, "lhs", 0);
-    check_adjs(check_i, check_j, x_rev, "rhs", 1);
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 0),
-                            index_min_max(ii, 1));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 1),
-                            index_min_max(ii, 0));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii),
-                            index_min_max(x.rows() + 1, 1),
-                            index_min_max(ii, 1));
-    test_throw_out_of_range(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 1),
-                            index_min_max(x.rows() + 1, 1));
-    // We don't want to go out of bounds when making the eigen block.
-    auto ii_range_high = ii == 5 ? 4 : ii + 1;
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii - 1, ii),
-                           index_min_max(ii, 1), index_min_max(ii, 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii_range_high, ii),
-                           index_min_max(ii, 1), index_min_max(ii, 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii - 1),
-                           index_min_max(ii, 1), index_min_max(ii, 1));
-    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii_range_high),
-                           index_min_max(ii, 1), index_min_max(ii, 1));
+    test_throw_invalid_arg(x, x_rev.block(0, 0, ii, ii), index_min_max(ii, 1),
+                           index_min_max(ii, 1));
     stan::math::recover_memory();
   }
 }
@@ -1608,6 +1509,26 @@ TEST_F(VarAssign, nil_matrix) {
   nil_matrix<double>();
 }
 
+template <typename RhsScalar>
+void size_zero_matrix() {
+  using stan::math::var_value;
+  using stan::model::test::check_adjs;
+  using stan::model::test::conditionally_generate_linear_var_matrix;
+  using stan::model::test::conditionally_generate_linear_var_vector;
+
+  auto x = conditionally_generate_linear_var_matrix(0, 0);
+  auto y = conditionally_generate_linear_var_matrix<RhsScalar>(0, 0);
+  assign(x, y, "");
+  assign(x, y, "", index_max(-1));
+  assign(x, y, "", index_min_max(1, -1));
+  assign(x, y, "", index_min(1), index_min_max(1, -1));
+}
+
+TEST_F(VarAssign, size_zero_matrix) {
+  size_zero_matrix<stan::math::var>();
+  size_zero_matrix<double>();
+}
+
 namespace stan {
 namespace model {
 namespace test {
